@@ -6,10 +6,11 @@ package Net::Server::Coro;
 use vars qw($VERSION);
 use EV;
 use Coro;
+use Coro::Specific;
 use base qw(Net::Server);
 use Net::Server::Proto::Coro;
 
-$VERSION = 0.6;
+$VERSION = 0.7;
 
 =head1 NAME
 
@@ -79,6 +80,11 @@ sub post_bind_hook {
     delete $prop->{select};
 
     $prop->{sock} = [ map { $self->make_coro_socket($_) } @{ $prop->{sock} } ];
+
+    # set up coro::specific variables
+    foreach my $key (qw(client sockaddr sockport peeraddr peerport)) {
+        tie $prop->{$key}, Coro::Specific::;
+    }
 }
 
 =head2 make_coro_socket SOCKET
@@ -115,14 +121,16 @@ sub coro_instance {
     $Coro::current->desc("Active connection");
     $prop->{client} = $fh;
     $self->run_client_connection;
-    close $fh;
 }
 
-# We override this to do nothing, or Net::Server closes
-# $self->{server}{client} -- which is he most recent connection, not
-# necessarily the current connection.  We also don't want the
-# STDERR/STDOUT redirection.
-sub post_process_request {}
+sub post_accept_hook {
+    my $self = shift;
+    my $prop = $self->{server};
+    my $sock = $prop->{client};
+
+    ($prop->{sockaddr}, $prop->{sockport}) = ($sock->sockhost, $sock->sockport);
+    ($prop->{peeraddr}, $prop->{peerport}) = ($sock->peerhost, $sock->peerport);
+}
 
 =head2 loop
 
@@ -183,7 +191,7 @@ sub loop {
 
 =head2 server_cert [PATH]
 
-Gets or sets the path fo the SSL certificate used by the server.
+Gets or sets the path of the SSL certificate used by the server.
 
 =cut
 
@@ -199,7 +207,7 @@ sub server_cert {
 
 =head2 server_key [PATH]
 
-Gets or sets the path fo the SSL key file used by the server.
+Gets or sets the path of the SSL key file used by the server.
 
 =cut
 
@@ -219,6 +227,11 @@ L<Coro>, L<EV>, L<Net::Server>
 
 =head1 BUGS AND LIMITATIONS
 
+The client filehandle, socket, and peer information all use
+L<Coro::Specific> in order to constrain their information to their
+coroutine.  Attempting to access them from a different coroutine will
+yield possibly unexpected results.
+
 Generally, all those of L<Coro>.  Please report any bugs or feature
 requests specific to L<Net::Server::Coro> to
 C<bug-net-server-coro@rt.cpan.org>, or through the web interface at
